using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;

using Infragistics.Win;
using Infragistics.Win.UltraWinGrid;
using MediaPlayerCommon;

using Microsoft.Practices.CompositeUI.EventBroker;
using Microsoft.Practices.CompositeUI.SmartParts;
using Microsoft.Practices.CompositeUI;

namespace PlaylistAddIn
{
    [SmartPart]
    public partial class PlaylistView : UserControl, ICanClose, ISmartPartInfoProvider
    {
        public PlaylistView()
        {
            InitializeComponent();

            //take care of some initializations after the constructor call
            this.ultraGridPlaylist.DataSource = _mediaCollection;
            this.cboExtension.SelectedIndex = 0;
        }

        private WorkItem _workItem;

        [ServiceDependency]
        public WorkItem WorkItem
        {
            set { _workItem = value; }
        }

        #region Event: MediaCollectionInitialized
        
        [EventPublication("event://MediaCollectionInitialized", PublicationScope.Global)]
        public event EventHandler<MediaCollectionEventArgs> MediaCollectionInitialized;
        
        /// <summary>
        /// This is the event raising method
        /// </summary>
        /// <param name="e"></param>
        protected void onMediaCollectionInitialized(MediaCollectionEventArgs e)
        {
            if (MediaCollectionInitialized != null)
            {
                MediaCollectionInitialized(this, e);
            }
        }

        #endregion

        #region Event: PlayCommandRequested

        [EventPublication("event://PlayCommandRequested", PublicationScope.Global)]
        public event EventHandler<MediaEventArgs> PlayCommandRequested;

        protected void onPlayCommandRequested(MediaEventArgs e)
        {
            if (PlayCommandRequested != null)
            {
                PlayCommandRequested(this, e);
            }
        }

        #endregion

        private List<MediaFile> _mediaCollection = new List<MediaFile>();

        /// <summary>
        /// This method simply gets the media files and assigns the data model to the Playlist
        /// </summary>
        public void SetPlaylistDataSource(string Path){
            _mediaCollection.AddRange( MediaManager.GetMediaFiles(Path, @"*." + cboExtension.Text) );
            
            this.ultraGridPlaylist.Rows.Refresh(RefreshRow.ReloadData);

            //we then use the event raising method, passing in the media collection.
            //cab will automatically allow any event subscribers to handle this event
            this.onMediaCollectionInitialized(new MediaCollectionEventArgs(_mediaCollection));
        }

        /// <summary>
        /// Here we will cause the Media Player to play the media represented
        /// by this row
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ultraGridPlaylist_DoubleClickRow(object sender, Infragistics.Win.UltraWinGrid.DoubleClickRowEventArgs e)
        {
            this.onPlayCommandRequested(new MediaEventArgs(e.Row.Index, e.Row.ListObject as MediaFile));
        }

        /// <summary>
        /// In this event, we assign the data filter to the one and only column in the playlist
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ultraGridPlaylist_InitializeLayout(object sender, InitializeLayoutEventArgs e)
        {
            e.Layout.Bands[0].Columns[0].Editor.DataFilter = new PlaylistDataFilter();
            e.Layout.Bands[0].HeaderVisible = false;
        }

        private void ultraButton1_Click(object sender, EventArgs e)
        {
            if (this.folderBrowserDialog1.ShowDialog() == DialogResult.OK)
            {
                this.SetPlaylistDataSource(this.folderBrowserDialog1.SelectedPath);
                this.folderBrowserDialog1.Reset();
            }
        }

        private void btnClearPlaylist_Click(object sender, EventArgs e)
        {
            _mediaCollection.Clear();
            this.ultraGridPlaylist.Rows.Refresh(RefreshRow.RefreshDisplay);
            this.onMediaCollectionInitialized(new MediaCollectionEventArgs(_mediaCollection));
        }

        [EventSubscription("event://CurrentMediaItemChanged")]
        public void CurrentMediaItemChanged(object Sender, MediaEventArgs e)
        {
            if (e.MediaIndex > -1 && this.ultraGridPlaylist.Rows.Count > 0)
            {
                this.ultraGridPlaylist.Selected.Rows.Clear();
                this.ultraGridPlaylist.Rows[e.MediaIndex].Activate();
                this.ultraGridPlaylist.Rows[e.MediaIndex].Selected = true;
            }
        }

        public ISmartPartInfo GetSmartPartInfo(Type smartPartInfoType)
        {
            // Implement ISmartPartInfoProvider in the containing smart part. Required in order for contained smart part infos to work.
            Microsoft.Practices.CompositeUI.SmartParts.ISmartPartInfoProvider ensureProvider = this;
            return this.infoProvider.GetSmartPartInfo(smartPartInfoType);

        }


        #region ICanClose Members

        bool ICanClose.CanClose
        {
            get
            {
                if (MessageBox.Show("Want to close the view?", "Confirm", MessageBoxButtons.OKCancel, MessageBoxIcon.Asterisk) == DialogResult.Cancel) return true;
                return false;                
            }

        }

        #endregion

    }

    /// <summary>
    /// this Data Filter is needed to transform what the data looks like in the Grid
    /// </summary>
    public class PlaylistDataFilter :IEditorDataFilter
    {

        public object Convert(Infragistics.Win.EditorDataFilterConvertArgs conversionArgs)
        {
            object theValue = null;

            switch (conversionArgs.Direction)
            {
                case ConversionDirection.OwnerToEditor:
                    conversionArgs.Handled = true;
                    theValue = ((MediaFile)((UltraGridCell)conversionArgs.Context).Row.ListObject).FileInfo.Name;
                    break;
                default:
                    theValue = conversionArgs.Value;
                    break;
            }
            return theValue;
        }

    }
}
